/**
 * easyWSDL - easyWSDL toolbox Platform.
 * Copyright (c) 2008,  eBM Websourcing
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.ow2.easywsdl.extensions.wsdl4complexwsdl.impl;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Description;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Document;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.ImportedDocuments;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlElement;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlException;
import org.ow2.easywsdl.schema.SchemaFactory;
import org.ow2.easywsdl.schema.api.Schema;
import org.ow2.easywsdl.schema.api.SchemaException;
import org.ow2.easywsdl.wsdl.WSDLFactory;
import org.ow2.easywsdl.wsdl.api.Binding;
import org.ow2.easywsdl.wsdl.api.Endpoint;
import org.ow2.easywsdl.wsdl.api.Import;
import org.ow2.easywsdl.wsdl.api.Include;
import org.ow2.easywsdl.wsdl.api.InterfaceType;
import org.ow2.easywsdl.wsdl.api.Service;
import org.ow2.easywsdl.wsdl.api.Types;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfDescription;
import org.ow2.easywsdl.wsdl.decorator.DecoratorDescriptionImpl;


/**
 * @author Nicolas Salatge - eBM WebSourcing
 */
public class DescriptionImpl extends
DecoratorDescriptionImpl<Service, Endpoint, Binding, InterfaceType, Include, Import, Types> implements
Description {

	/**
	 *
	 */
	private static final long serialVersionUID = 1L;

	private WSDL4ComplexWsdlElement elmt = null;

	@SuppressWarnings("unchecked")
	public DescriptionImpl(final AbsItfDescription wsdl) throws WSDLException {
		super(wsdl, null);

		AbsItfDescription firstWsdl = wsdl;
		if(wsdl instanceof DecoratorDescriptionImpl) {
			firstWsdl = ((DecoratorDescriptionImpl)wsdl).getFirstDescription();
		}

		if(firstWsdl instanceof org.ow2.easywsdl.wsdl.api.Description) {
			this.elmt = new WSDL4ComplexWsdlElementImpl<AbsItfDescription>(firstWsdl);
		} else {
			throw new WSDL4ComplexWsdlException("Impossible to find the correct first description");
		}
	}

	public ImportedDocuments getImportedDocuments() {
		return this.elmt.getImportedDocuments();
	}

	public void setImportedDocuments(ImportedDocuments docs) throws WSDL4ComplexWsdlException {
		this.elmt.setImportedDocuments(docs);
	}

	public void addImportedDocumentsInWsdl() throws WSDL4ComplexWsdlException {
		//		this.getSchemaLocation().put("http://petals.ow2.org/wsdlExtensions" , "wsdlExtensions.xsd");
		//		this.addNamespace("petalsWsdlExtensions", "http://petals.ow2.org/wsdlExtensions");

		ImportedDocuments imptDocs = new ImportedDocumentsImpl(new org.ow2.easywsdl.extensions.complexwsdl.ImportedDocuments(), this);
		Map<String, Document> documents = getAllWsdlDocuments(this, imptDocs);



		imptDocs.addAllDocuments(documents.values());

		this.elmt.setImportedDocuments(imptDocs);
	}

	private Map<String, Document> getAllWsdlDocuments(AbsItfDescription description, ImportedDocuments imptDocs) throws WSDL4ComplexWsdlException {
		Map<String, Document> res = new HashMap<String, Document>();

		List<Import> wsdlImports = description.getImports();
		List<Include> wsdlIncludes = description.getIncludes();
		List<org.ow2.easywsdl.schema.api.Import> schemaImports = null;
		if(description.getTypes() != null) {
			schemaImports = description.getTypes().getImportedSchemas();
		}
		Map<String, String> map = description.getSchemaLocation();
		
		// TODO: Add schema location import

		// Insert all wsdl imports in document
		for(Import imptWsdl: wsdlImports) {
			String internalLocation = this.createInternalLocation(imptWsdl.getLocationURI());
			if(!res.containsKey(internalLocation)) {
				if(imptWsdl.getDescription() != null) {
					Document doc = new DocumentImpl(new org.ow2.easywsdl.extensions.complexwsdl.Document(), imptDocs);

					doc.setName(internalLocation);

					doc.setImportedDescription(imptWsdl.getDescription());

					res.put(internalLocation, doc);

					// change location
					imptWsdl.setLocationURI(internalLocation);

					// add recursive import
					res.putAll(getAllWsdlDocuments(imptWsdl.getDescription(), imptDocs));
				}
			}

		}

		// Insert all wsdl includes in document
		for(Include inclWsdl: wsdlIncludes) {
			String internalLocation = this.createInternalLocation(inclWsdl.getLocationURI());
			if(!res.containsKey(internalLocation)) {
				if(inclWsdl.getDescription() != null) {
					Document doc = new DocumentImpl(new org.ow2.easywsdl.extensions.complexwsdl.Document(), imptDocs);
					doc.setName(internalLocation);
					doc.setImportedDescription(inclWsdl.getDescription());
					res.put(internalLocation, doc);

					// change location
					inclWsdl.setLocationURI(internalLocation);

					// add recursive includes
					res.putAll(getAllWsdlDocuments(inclWsdl.getDescription(), imptDocs));
				}
			}
		}

		// Insert all schema imports in document
		if(description.getTypes() != null) {
			if(schemaImports != null) {
				for(org.ow2.easywsdl.schema.api.Import imptSchema: schemaImports) {
					String internalLocation = this.createInternalLocation(imptSchema.getLocationURI());
					if(!res.containsKey(internalLocation)) {
						if(imptSchema.getSchema() != null) {
							Document doc = new DocumentImpl(new org.ow2.easywsdl.extensions.complexwsdl.Document(), imptDocs);

							doc.setName(internalLocation);
							doc.setImportedSchema(imptSchema.getSchema());
							res.put(internalLocation, doc);

							// change location
							imptSchema.setLocationURI(internalLocation);

							// add recursive schema imports 
							res.putAll(getAllSchemaDocuments(imptSchema.getSchema(), imptDocs));
						}
					}

				}
			}
			for(Schema schema: (List<Schema>)description.getTypes().getSchemas()) {
				// add recursive schema imports 
				res.putAll(getAllSchemaDocuments(schema, imptDocs));
			}
		}
		return res;
	}

	private Map<String, Document> getAllSchemaDocuments(
			Schema schema, ImportedDocuments imptDocs) throws WSDL4ComplexWsdlException {
		Map<String, Document> res = new HashMap<String, Document>();

		for(org.ow2.easywsdl.schema.api.Import imptSchema: schema.getImports()) {
			String internalLocation = this.createInternalLocation(imptSchema.getLocationURI());
			if(!res.containsKey(internalLocation)) {
				if(imptSchema.getSchema() != null) {
					Document doc = new DocumentImpl(new org.ow2.easywsdl.extensions.complexwsdl.Document(), imptDocs);

					doc.setName(internalLocation);
					doc.setImportedSchema(imptSchema.getSchema());
					res.put(internalLocation, doc);

					// change location
					imptSchema.setLocationURI(internalLocation);

					// add recursive schema imports 
					res.putAll(getAllSchemaDocuments(imptSchema.getSchema(), imptDocs));
				}
			}

		}

		for(org.ow2.easywsdl.schema.api.Include inclSchema: schema.getIncludes()) {
			String internalLocation = this.createInternalLocation(inclSchema.getLocationURI());
			if(!res.containsKey(internalLocation)) {
				if(inclSchema.getSchema() != null) {
					Document doc = new DocumentImpl(new org.ow2.easywsdl.extensions.complexwsdl.Document(), imptDocs);

					doc.setName(internalLocation);
					doc.setImportedSchema(inclSchema.getSchema());
					res.put(internalLocation, doc);

					// change location
					inclSchema.setLocationURI(internalLocation);

					// add recursive schema imports 
					res.putAll(getAllSchemaDocuments(inclSchema.getSchema(), imptDocs));
				}
			}
		}

		return res;
	}

	private String createInternalLocation(String currentLocation) {
		String internalLocation = null;
		if((currentLocation != null)&&(!currentLocation.startsWith("file:///description/importedDocuments/document/"))) {
			String fileName = null;
			if((currentLocation.endsWith(".xsd"))||(currentLocation.endsWith(".wsdl"))) {
				if(currentLocation.indexOf("/") > 0) {
					fileName = currentLocation.substring(currentLocation.indexOf("/"));
				} else {
					fileName = currentLocation;
				}
			} else {
				fileName = "unnamedDocument" + this.getImportedDocuments().getDocuments().size();
			}
			internalLocation = "file:///description/importedDocuments/document/" + fileName;
		} else {
			internalLocation = currentLocation;
		}
		return internalLocation;
	}

	public Map<URI, org.w3c.dom.Document> deleteImportedDocumentsInWsdl(String externalPath) throws WSDL4ComplexWsdlException {
		Map<URI, org.w3c.dom.Document> imports = new HashMap<URI, org.w3c.dom.Document>();
		try {
			// The following has to be done only when the importedDocuments is not null ;-)
			if (this.getImportedDocuments() != null){
			
			// change location in main description
			changeImportedLocationInWsdl(this, externalPath);

			// get All imports
			for(Document doc : this.getImportedDocuments().getDocuments()) {
				String externalLocation = this.createExternalLocation(externalPath, doc.getName());
				org.w3c.dom.Document document = null;

				if(doc.getImportedDescription() != null) {
					changeImportedLocationInWsdl(doc.getImportedDescription(), externalPath);
					document = WSDLFactory.newInstance().newWSDLWriter().getDocument(doc.getImportedDescription());
				} else if(doc.getImportedSchema() != null) {
					changeImportedLocationInSchema(doc.getImportedSchema(), externalPath);
					document = SchemaFactory.newInstance().newSchemaWriter().getDocument(doc.getImportedSchema());
				} 
				imports.put(new URI(externalLocation), document);
			}

			// remove imported documents
			this.setImportedDocuments(null);
			}
		} catch (URISyntaxException e) {
			throw new WSDL4ComplexWsdlException(e);
		} catch (WSDLException e) {
			throw new WSDL4ComplexWsdlException(e);
		} catch (SchemaException e) {
			throw new WSDL4ComplexWsdlException(e);
		}		
		return imports;
	}

	private void changeImportedLocationInSchema(Schema schema,
			String externalPath) {
		
		// TODO: change schemaLocation import
		
		// change location in schema import
		for(org.ow2.easywsdl.schema.api.Import impt: schema.getImports()) {
			impt.setLocationURI(this.createExternalLocation(externalPath, impt.getLocationURI()));
		}

		// change location in schema include
		for(org.ow2.easywsdl.schema.api.Include incl: schema.getIncludes()) {
			incl.setLocationURI(this.createExternalLocation(externalPath, incl.getLocationURI()));
		}
	}

	private void changeImportedLocationInWsdl(AbsItfDescription description,
			String externalPath) {
		
		// TODO: change schemaLocation import

		List<Import> wsdlImports = description.getImports();
		List<Include> wsdlIncludes = description.getIncludes();
		List<org.ow2.easywsdl.schema.api.Import> schemaImports = null;
		if(description.getTypes() != null) {
			schemaImports = description.getTypes().getImportedSchemas();
		}

		// change location in wsdl import
		for(Import impt : wsdlImports) {
			impt.setLocationURI(this.createExternalLocation(externalPath, impt.getLocationURI()));
		}

		// change location in wsdl include
		for(Include incl : wsdlIncludes) {
			incl.setLocationURI(this.createExternalLocation(externalPath, incl.getLocationURI()));
		}

		// change location in schema import
		if(schemaImports != null) {
			for(org.ow2.easywsdl.schema.api.Import impt: schemaImports) {
				impt.setLocationURI(this.createExternalLocation(externalPath, impt.getLocationURI()));
			}
		}

		// change location all schema imports in document
		if(description.getTypes() != null) {
			for(Schema schema: (List<Schema>)description.getTypes().getSchemas()) {
				this.changeImportedLocationInSchema(schema, externalPath);
			}
		}		
	}

	private String createExternalLocation(String externalPath, String currentLocation) {
		String externalLocation = null;
		if(currentLocation.startsWith("file:///description/importedDocuments/document/")) {
			externalLocation = currentLocation.replace("file:///description/importedDocuments/document/", externalPath);
		} else {
			externalLocation = currentLocation;
		}
		return externalLocation;
	}

}
