/**
 * easyWSDL - easyWSDL toolbox Platform.
 * Copyright (c) 2008,  eBM Websourcing
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.ow2.easywsdl.extensions.wsdl4complexwsdl.impl;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Map;
import java.util.logging.Logger;

import javax.xml.transform.dom.DOMSource;

import org.jdom.input.DOMBuilder;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Document;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.ImportedDocuments;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlException;
import org.ow2.easywsdl.schema.SchemaFactory;
import org.ow2.easywsdl.schema.api.Schema;
import org.ow2.easywsdl.schema.api.SchemaException;
import org.ow2.easywsdl.schema.api.abstractElmt.AbstractSchemaElementImpl;
import org.ow2.easywsdl.schema.impl.SchemaImpl;
import org.ow2.easywsdl.schema.impl.SchemaWriterImpl;
import org.ow2.easywsdl.wsdl.WSDLFactory;
import org.ow2.easywsdl.wsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.WSDLImportException;
import org.ow2.easywsdl.wsdl.api.WSDLReader.FeatureConstants;
import org.ow2.easywsdl.wsdl.api.abstractElmt.AbstractWSDLElementImpl;
import org.ow2.easywsdl.wsdl.org.w3.ns.wsdl.DescriptionType;
import org.ow2.easywsdl.wsdl.org.xmlsoap.schemas.wsdl.TDefinitions;

/** 
 * @author Nicolas Salatge - eBM WebSourcing
 */
public class DocumentImpl extends AbstractWSDLElementImpl<org.ow2.easywsdl.extensions.complexwsdl.Document> implements Document {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private static Logger log = Logger.getLogger(DocumentImpl.class.getName());

	private Description imptDesc = null;

	private Schema imptSchema = null;

	public DocumentImpl(final org.ow2.easywsdl.extensions.complexwsdl.Document doc, final ImportedDocuments docs) throws WSDL4ComplexWsdlException {
		super(doc, (AbstractWSDLElementImpl) docs);
		try {
			if(doc.getDefinition() != null) {
				Map<FeatureConstants, Object> features = WSDLFactory.newInstance().newWSDLReader().getFeatures();
				features.put(FeatureConstants.IMPORT_DOCUMENTS, false);

				org.w3c.dom.Document dom = new org.ow2.easywsdl.wsdl.impl.wsdl11.WSDLWriterImpl().convertWSDL11Definition2DOMElement(doc.getDefinition());
				DOMBuilder builder = new DOMBuilder();
				org.jdom.Document jdomDocument = builder.build(dom);
				
				imptDesc = new org.ow2.easywsdl.wsdl.impl.wsdl11.DescriptionImpl(new URI("."), doc.getDefinition(), jdomDocument, features, (Map<URI, DOMSource>)null, 6);
			} else if(doc.getDescription() != null) {
				Map<FeatureConstants, Object> features = WSDLFactory.newInstance().newWSDLReader().getFeatures();
				features.put(FeatureConstants.IMPORT_DOCUMENTS, false);
				
				org.w3c.dom.Document dom = new org.ow2.easywsdl.wsdl.impl.wsdl20.WSDLWriterImpl().convertWSDL20Description2DOMElement(doc.getDescription());
				DOMBuilder builder = new DOMBuilder();
				org.jdom.Document jdomDocument = builder.build(dom);
				
				imptDesc = new org.ow2.easywsdl.wsdl.impl.wsdl20.DescriptionImpl(new URI("."), doc.getDescription(), jdomDocument, features, (Map<URI, DOMSource>)null, 6);
			} else if(doc.getSchema() != null) {
				Map<org.ow2.easywsdl.schema.api.SchemaReader.FeatureConstants, Object> features = SchemaFactory.newInstance().newSchemaReader().getFeatures();
				features.put(org.ow2.easywsdl.schema.api.SchemaReader.FeatureConstants.IMPORT_DOCUMENTS, false);

				org.w3c.dom.Document dom = new SchemaWriterImpl().convertSchema2DOMElement(doc.getSchema());
				DOMBuilder builder = new DOMBuilder();
				org.jdom.Document jdomDocument = builder.build(dom);

				imptSchema = new SchemaImpl(new URI("."), doc.getSchema(), jdomDocument, features, (Map<URI, DOMSource>)null);
			}
		} catch (WSDLException e) {
			throw new WSDL4ComplexWsdlException(e);
		} catch (WSDLImportException e) {
			throw new WSDL4ComplexWsdlException(e);
		} catch (URISyntaxException e) {
			throw new WSDL4ComplexWsdlException(e);
		} catch (SchemaException e) {
			throw new WSDL4ComplexWsdlException(e);
		}
	}

	public String getName() {
		return this.model.getName();
	}

	public void setName(String name) {
		this.model.setName(name);
	}

	public Description getImportedDescription() {
		return this.imptDesc;
	}

	public Schema getImportedSchema() {
		return this.imptSchema;
	}

	public void setImportedDescription(Description desc) {
		this.imptDesc = desc;
		if(((AbstractWSDLElementImpl)desc).getModel() instanceof TDefinitions) {
			this.model.setDefinition((TDefinitions) ((AbstractWSDLElementImpl)desc).getModel());
		} else if(((AbstractWSDLElementImpl)desc).getModel() instanceof DescriptionType) {
			this.model.setDescription((DescriptionType) ((AbstractWSDLElementImpl)desc).getModel());
		} 
	}

	public void setImportedSchema(Schema schema) {
		this.imptSchema = schema;
		this.model.setSchema((org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Schema) ((AbstractSchemaElementImpl)schema).getModel());
	}


}
