/**
 * easyWSDL - easyWSDL toolbox Platform.
 * Copyright (c) 2008,  eBM Websourcing
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.ow2.easywsdl.extensions.wsdl4complexwsdl.impl.inout;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Map;

import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.dom.DOMSource;

import org.ow2.easywsdl.extensions.complexwsdl.ImportedDocuments;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.WSDL4ComplexWsdlFactory;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Description;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlException;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlReader;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.impl.WSDL4ComplexWsdlJAXBContext;
import org.ow2.easywsdl.schema.SchemaFactory;
import org.ow2.easywsdl.schema.api.SchemaException;
import org.ow2.easywsdl.wsdl.WSDLFactory;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.WSDLReader;
import org.ow2.easywsdl.wsdl.api.WSDLReader.FeatureConstants;
import org.ow2.easywsdl.wsdl.api.abstractElmt.AbstractWSDLReaderImpl;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * @author Nicolas Salatge - eBM WebSourcing
 */
public class WSDL4ComplexWsdlReaderImpl implements WSDL4ComplexWsdlReader {

	WSDLReader reader = null;

	/**
	 * The JAXB unique unmarshaller.
	 */
	private Unmarshaller unmarshaller = null;

	public WSDL4ComplexWsdlReaderImpl() throws WSDL4ComplexWsdlException {
		try {
			WSDLFactory factory = WSDLFactory.newInstance();
			factory.setJAXBObjectFactoryList(WSDL4ComplexWsdlJAXBContext.getDefaultObjectFactories());
			this.reader = factory.newWSDLReader();
			this.reader.getFeatures().put(FeatureConstants.IMPORT_DOCUMENTS, false);

			WSDL4ComplexWsdlJAXBContext.createDefaultContext();
			this.unmarshaller = WSDL4ComplexWsdlJAXBContext.getJaxbContext().createUnmarshaller();
		} catch (final WSDLException e) {
			throw new WSDL4ComplexWsdlException(e);
		} catch (JAXBException e) {
			throw new WSDL4ComplexWsdlException(e);
		}
	}

	public Object getFeature(final FeatureConstants name) {
		return this.reader.getFeature(name);
	}

	public Map<FeatureConstants, Object> getFeatures() {
		return this.reader.getFeatures();
	}

	public void setFeatures(final Map<FeatureConstants, Object> features) {
		((AbstractWSDLReaderImpl) this.reader).setFeatures(features);
	}

	public Description readWSDL4ComplexWSDL(final URI wsdlURI) throws WSDL4ComplexWsdlException {
		Description res = null;
		try {
			org.ow2.easywsdl.wsdl.api.Description desc = this.reader.readWSDL(wsdlURI);
			res = WSDL4ComplexWsdlFactory.newInstance().addComplexWsdlElmt2Description(desc);

			if(res.getImportedDocuments() != null) {
				Map<URI, Document> imports = new HashMap<URI, Document>();
				for(org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Document doc: res.getImportedDocuments().getDocuments()) {
					if(doc.getImportedDescription() != null) {
						Document source = WSDLFactory.newInstance().newWSDLWriter().getDocument(doc.getImportedDescription());
						imports.put(new URI(doc.getName()), source);
					} else if(doc.getImportedSchema() != null) {
						Document source = SchemaFactory.newInstance().newSchemaWriter().getDocument(doc.getImportedSchema());
						imports.put(new URI(doc.getName()), source);
					}
				}
				this.reader.getFeatures().put(FeatureConstants.IMPORT_DOCUMENTS, true);
				desc = this.reader.readWSDL(WSDL4ComplexWsdlFactory.newInstance().newWSDLWriter().getDocument(res), imports);
				res = WSDL4ComplexWsdlFactory.newInstance().addComplexWsdlElmt2Description(desc);
				this.reader.getFeatures().put(FeatureConstants.IMPORT_DOCUMENTS, false);

			} else {
				this.reader.getFeatures().put(FeatureConstants.IMPORT_DOCUMENTS, true);
				desc = this.reader.readWSDL(wsdlURI);
				res = WSDL4ComplexWsdlFactory.newInstance().addComplexWsdlElmt2Description(desc);
				this.reader.getFeatures().put(FeatureConstants.IMPORT_DOCUMENTS, false);
			}
		} catch (final WSDLException e) {
			throw new WSDL4ComplexWsdlException(e);
		} catch (SchemaException e) {
			throw new WSDL4ComplexWsdlException(e);
		} catch (URISyntaxException e) {
			throw new WSDL4ComplexWsdlException(e);
		}
		return res;
	}

	public Description readWSDL4ComplexWSDL(final Document wsdlDocument) throws WSDL4ComplexWsdlException {
		Description res = null;
		try {
			org.ow2.easywsdl.wsdl.api.Description desc = this.reader.readWSDL(wsdlDocument);
			res = WSDL4ComplexWsdlFactory.newInstance().addComplexWsdlElmt2Description(desc);

			if(res.getImportedDocuments() != null) {
				Map<URI, Document> imports = new HashMap<URI, Document>();
				for(org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Document doc: res.getImportedDocuments().getDocuments()) {
					if(doc.getImportedDescription() != null) {
						Document source = WSDLFactory.newInstance().newWSDLWriter().getDocument(doc.getImportedDescription());
						imports.put(new URI(doc.getName()), source);
					} else if(doc.getImportedSchema() != null) {
						Document source = SchemaFactory.newInstance().newSchemaWriter().getDocument(doc.getImportedSchema());
						imports.put(new URI(doc.getName()), source);
					}
				}
				this.reader.getFeatures().put(FeatureConstants.IMPORT_DOCUMENTS, true);
				desc = this.reader.readWSDL(wsdlDocument, imports);
				res = WSDL4ComplexWsdlFactory.newInstance().addComplexWsdlElmt2Description(desc);
				this.reader.getFeatures().put(FeatureConstants.IMPORT_DOCUMENTS, false);
			} else {
				this.reader.getFeatures().put(FeatureConstants.IMPORT_DOCUMENTS, true);
				desc = this.reader.readWSDL(wsdlDocument);
				res = WSDL4ComplexWsdlFactory.newInstance().addComplexWsdlElmt2Description(desc);
				this.reader.getFeatures().put(FeatureConstants.IMPORT_DOCUMENTS, false);
			}

		} catch (final WSDLException e) {
			throw new WSDL4ComplexWsdlException(e);
		} catch (URISyntaxException e) {
			throw new WSDL4ComplexWsdlException(e);
		} catch (SchemaException e) {
			throw new WSDL4ComplexWsdlException(e);
		}
		return res;
	}

	public Description readWSDL4ComplexWsdl(final URI wsdlURI, final DOMSource inputSource) throws WSDL4ComplexWsdlException {
		Description res = null;
		try {
			final org.ow2.easywsdl.wsdl.api.Description desc = this.reader.readWSDL(wsdlURI,
					inputSource);
			res = WSDL4ComplexWsdlFactory.newInstance().addComplexWsdlElmt2Description(desc);
		} catch (final WSDLException e) {
			throw new WSDL4ComplexWsdlException(e);
		}
		return res;
	}

	public void setFeature(final FeatureConstants name, final Object value) throws WSDLException {
		this.reader.setFeature(name, value);
	}


	public JAXBElement<ImportedDocuments> convertElement2ImportedDocuments(final Element imptDocs)
	throws WSDLException {
		JAXBElement<ImportedDocuments> res = null;
		try {
			res = this.unmarshaller.unmarshal(imptDocs, ImportedDocuments.class);
		} catch (final JAXBException e) {
			throw new WSDLException(
					"Failed to build Java bindings from WSDL descriptor XML document", e);
		}
		return res;
	}
}
